module.exports={A:{A:{"2":"fB","8":"I D E F A","129":"B"},B:{"1":"R S T U V W X P Y Z G","129":"C J K L M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB iB jB","129":"H a I D E F A B C J K L M N O b c d e f"},D:{"1":"0 1 2 3 4 5 6 7 8 9 p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"H a I D","129":"E F A B C J K L M N O b c d e f g h i j k l m n o"},E:{"1":"E F A B C J K rB cB TB UB sB tB uB","2":"H a nB bB","129":"I D oB pB qB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F B vB wB xB yB TB dB zB","129":"C L M N O UB"},G:{"1":"E 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC","2":"bB 0B eB 1B 2B 3B"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"1":"A","2":"D"},K:{"1":"C Q UB","2":"A B TB dB"},L:{"1":"G"},M:{"1":"P"},N:{"8":"A","129":"B"},O:{"129":"QC"},P:{"1":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"1":"aC"},R:{"1":"bC"},S:{"129":"cC"}},B:6,C:"WebGL - 3D Canvas graphics"};
