module.exports={A:{A:{"1":"B","2":"I D E F A fB"},B:{"2":"C J K L M N O R S T U V W X P Y Z G"},C:{"1":"0 1 2 3 4 5 6 J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z","2":"7 8 9 gB VB H a I D E F A B C AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G iB jB"},D:{"1":"0 1 2 3 4 5 6 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z","2":"7 8 9 AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB"},E:{"1":"E F A B C rB cB TB","2":"H a I D nB bB oB pB qB","129":"J K UB sB tB uB"},F:{"1":"0 L M N O b c d e f g h i j k l m n o p q r s t u v y UB","2":"1 2 3 4 5 6 7 8 9 F B C w x z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB vB wB xB yB TB dB zB"},G:{"1":"E 4B 5B 6B 7B 8B 9B AC BC","2":"bB 0B eB 1B 2B 3B","257":"CC DC EC FC GC HC IC"},H:{"2":"JC"},I:{"1":"VB H NC eB OC PC","2":"G KC LC MC"},J:{"2":"D A"},K:{"1":"UB","2":"A B C Q TB dB"},L:{"2":"G"},M:{"2":"P"},N:{"1":"B","2":"A"},O:{"2":"QC"},P:{"1":"H","2":"RC SC TC UC VC cB WC XC YC ZC"},Q:{"2":"aC"},R:{"16":"bC"},S:{"1":"cC"}},B:7,C:"SPDY protocol"};
