module.exports={A:{A:{"2":"I D E F A fB","2052":"B"},B:{"1":"C J K L M N O R S T U V W X P Y Z G"},C:{"1":"0 1 2 3 4 5 6 7 8 9 s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","132":"gB VB H a I D E F A B C iB jB","260":"J K L M N O b c d e f g h i j k l m n o p q r"},D:{"1":"5 6 7 8 9 AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","260":"H a I D E F A B C J K L M N O b c","772":"d e f g h i j k l m n o p q r s t u v w","1028":"0 1 2 3 4 x y z"},E:{"1":"A B C J K cB TB UB sB tB uB","260":"H a nB bB","772":"I D E F oB pB qB rB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F vB","132":"B wB xB yB TB dB","644":"C zB UB","772":"L M N O b c d e f g h i j","1028":"k l m n o p q r"},G:{"1":"7B 8B 9B AC BC CC DC EC FC GC HC IC","260":"bB 0B eB","772":"E 1B 2B 3B 4B 5B 6B"},H:{"644":"JC"},I:{"1":"G","16":"KC LC","260":"MC","772":"VB H NC eB OC PC"},J:{"772":"D A"},K:{"1":"Q","132":"A B TB dB","644":"C UB"},L:{"1":"G"},M:{"1":"P"},N:{"1":"B","2":"A"},O:{"1":"QC"},P:{"1":"RC SC TC UC VC cB WC XC YC ZC","1028":"H"},Q:{"1":"aC"},R:{"1028":"bC"},S:{"1":"cC"}},B:6,C:"const"};
