module.exports={A:{A:{"1":"A B","2":"I D E F fB"},B:{"1":"R S T U V W X P Y Z G","2":"C J K L M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB iB jB","132":"H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s"},D:{"1":"0 1 2 3 4 5 6 7 8 9 i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"H","16":"a I D E d e f g h","132":"F A B C J K L M N O b c"},E:{"1":"C J K TB UB sB tB uB","2":"H a nB bB oB","132":"I D E F A B pB qB rB cB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F B C vB wB xB yB TB dB zB UB"},G:{"2":"2B 3B","132":"E 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC","514":"bB 0B eB 1B"},H:{"2":"JC"},I:{"2":"KC LC MC","260":"VB H NC eB","514":"G OC PC"},J:{"132":"A","260":"D"},K:{"2":"A B C TB dB UB","260":"Q"},L:{"260":"G"},M:{"2":"P"},N:{"514":"A","1028":"B"},O:{"2":"QC"},P:{"260":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"260":"aC"},R:{"260":"bC"},S:{"1":"cC"}},B:1,C:"accept attribute for file input"};
