module.exports={A:{A:{"2":"I D E F fB","6308":"A","6436":"B"},B:{"1":"R S T U V W X P Y Z G","6436":"C J K L M N O"},C:{"1":"LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u iB jB","2052":"0 1 2 3 4 5 6 7 8 9 v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB"},D:{"1":"MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"0 1 2 3 4 5 6 7 8 9 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB","8258":"JB KB LB"},E:{"1":"B C J K TB UB sB tB uB","2":"H a I D E nB bB oB pB qB","3108":"F A rB cB"},F:{"1":"HB IB JB KB LB MB NB OB PB QB RB SB","2":"0 1 2 3 4 5 6 7 8 9 F B C L M N O b c d e f g h i j k l m n o p q r s t u v w x y z vB wB xB yB TB dB zB UB","8258":"AB BB CB DB EB FB Q GB"},G:{"1":"9B AC BC CC DC EC FC GC HC IC","2":"E bB 0B eB 1B 2B 3B 4B","3108":"5B 6B 7B 8B"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"2":"A B C Q TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"2":"QC"},P:{"1":"cB WC XC YC ZC","2":"H RC SC TC UC VC"},Q:{"2":"aC"},R:{"2":"bC"},S:{"2052":"cC"}},B:4,C:"CSS Scroll Snap"};
