html {
   margin: 0px;
   height: 100%;
   width: 100%;
}

body {
   margin: 0px;
   min-height: 100%;
   width: 100%;
}


h1 {
    text-align: center;
}

#terminal-container {
    height: 100vh;
    margin: 0px;
    padding: 0px;
    z-index: 255;
    overflow: hidden;
}

#terminal-container .terminal {
    background-color: #000;
    color: #fafafa;
    padding: 7px;
    font-family: 'Ubuntu Mono', monospace;
    height:100%;
    font-size: 1em;
    line-height: 1 !important;
}

/* --- Scrollbar -- */

div.dark-scroll ::-webkit-scrollbar {
    background-color: #19232D;
    width: 14px;
    margin: 16px 2px 16px 2px;
    border: 1px solid #32414B;
    border-radius: 4px;
}

div.dark-scroll ::-webkit-scrollbar-thumb {
    background-color: #787878;
    border: 1px solid #32414B;
    min-height: 8px;
    border-radius: 10px;
    width: 1px;
}

div.dark-scroll ::-webkit-scrollbar-thumb:hover {
    background-color: #148CD2;
    border: 1px solid #148CD2;
    border-radius: 10px;
    min-height: 8px;
    width: 1px;
}

div.dark-scroll ::-webkit-scrollbar-button:increment {
    background-repeat: no-repeat;
    background-image: url(../images/down_arrow_disabled.png);
    background-position: center center;
}

div.dark-scroll ::-webkit-scrollbar-button:decrement {
    background-repeat: no-repeat;
    background-image: url(../images/up_arrow_disabled.png);
    background-position: center center;
}

div.dark-scroll ::-webkit-scrollbar-button:increment:hover {
    background-repeat: no-repeat;
    background-image: url(../images/down_arrow.png);
    background-position: center center;
}

div.dark-scroll ::-webkit-scrollbar-button:decrement:hover {
    background-repeat: no-repeat;
    background-image: url(../images/up_arrow.png);
    background-position: center center;
}
