module.exports={A:{A:{"2":"I D E F A fB","130":"B"},B:{"1":"Z G","16":"C J","260":"K L","1028":"R S T U V W X P Y","5124":"M N O"},C:{"2":"0 1 2 3 4 5 6 7 8 9 gB VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB iB jB","6148":"IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","6722":"DB EB WB FB XB Q GB HB"},D:{"1":"Z G kB lB mB","2":"0 1 2 3 4 5 6 7 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z","260":"8 9 AB BB CB DB EB","1028":"WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y"},E:{"2":"H a I D E F nB bB oB pB qB rB","1028":"tB uB","3076":"A B C J K cB TB UB sB"},F:{"2":"F B C L M N O b c d e f g h i j k l m n o p q r s t u vB wB xB yB TB dB zB UB","260":"0 1 v w x y z","1028":"2 3 4 5 6 7 8 9 AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB"},G:{"2":"E bB 0B eB 1B 2B 3B 4B 5B 6B","16":"7B","1028":"8B 9B AC BC CC DC EC FC GC HC IC"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"2":"A B C TB dB UB","1028":"Q"},L:{"1":"G"},M:{"6148":"P"},N:{"2":"A B"},O:{"2":"QC"},P:{"2":"H RC SC","1028":"TC UC VC cB WC XC YC ZC"},Q:{"1028":"aC"},R:{"2":"bC"},S:{"2":"cC"}},B:1,C:"Streams"};
