module.exports={A:{A:{"2":"I D E F A B fB"},B:{"1":"R S T U V W X P Y Z G","2":"C J K L M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB H a I D E F A B C J K iB jB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","16":"H a I D E F A B C J K"},E:{"1":"I D E F A B C J K oB pB qB rB cB TB UB sB tB uB","2":"H a nB bB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 C L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB zB UB","2":"F B vB wB xB yB TB dB"},G:{"1":"E 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC","16":"bB 0B eB"},H:{"1":"JC"},I:{"1":"G OC PC","16":"VB H KC LC MC NC eB"},J:{"16":"D A"},K:{"1":"C Q UB","2":"A B TB dB"},L:{"1":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"1":"QC"},P:{"1":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"1":"aC"},R:{"1":"bC"},S:{"1":"cC"}},B:4,C:"SVG vector-effect: non-scaling-stroke"};
