module.exports={A:{A:{"2436":"I D E F A B fB"},B:{"260":"N O","2436":"C J K L M","8196":"R S T U V W X P Y Z G"},C:{"2":"gB VB H a I D E F A B C J K L M N O b c d iB jB","772":"e f g h i j k l m n o p q r s t u v w","4100":"0 1 2 3 4 5 6 7 8 9 x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G"},D:{"2":"H a I D E F A B C","2564":"J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y","8196":"EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","10244":"0 1 2 3 4 5 6 7 8 9 z AB BB CB DB"},E:{"1":"C J K UB sB tB uB","16":"nB bB","2308":"A B cB TB","2820":"H a I D E F oB pB qB rB"},F:{"2":"F B vB wB xB yB TB dB zB","16":"C","516":"UB","2564":"L M N O b c d e f g h i j k l","8196":"1 2 3 4 5 6 7 8 9 AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","10244":"0 m n o p q r s t u v w x y z"},G:{"1":"BC CC DC EC FC GC HC IC","2":"bB 0B eB","2820":"E 1B 2B 3B 4B 5B 6B 7B 8B 9B AC"},H:{"2":"JC"},I:{"2":"VB H KC LC MC NC eB","260":"G","2308":"OC PC"},J:{"2":"D","2308":"A"},K:{"2":"A B C TB dB","16":"UB","1028":"Q"},L:{"8196":"G"},M:{"1028":"P"},N:{"2":"A B"},O:{"2":"QC"},P:{"2052":"RC SC","2308":"H","8196":"TC UC VC cB WC XC YC ZC"},Q:{"10244":"aC"},R:{"2052":"bC"},S:{"4100":"cC"}},B:5,C:"Synchronous Clipboard API"};
