body {
    font-family: helvetica, sans-serif, arial;
    font-size: 1em;
    color: #111;
}

h1 {
    text-align: center;
}

#terminal-container {
    width: 800px;
    height: 450px;
    margin: 0 auto;
    padding: 2px;
}

p {
    font-size: 0.9em;
    font-style: italic
}

#option-container {
    display: flex;
    justify-content: center;
}

.option-group {
    display: inline-block;
    padding-left: 20px;
    vertical-align: top;
}
