module.exports={A:{A:{"1":"B","2":"I D E fB","8":"F A"},B:{"1":"C J K L M N O R S T U V W X P Y Z G"},C:{"1":"0 1 2 3 4 5 6 7 8 9 K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB H a I D E F A B C J iB jB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"H a I D E F A B C J K L M N","33":"O b c d e f g h i"},E:{"1":"D E F A B C J K pB qB rB cB TB UB sB tB uB","2":"H a nB bB oB","33":"I"},F:{"1":"0 1 2 3 4 5 6 7 8 9 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F B C vB wB xB yB TB dB zB UB"},G:{"1":"E 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC","2":"bB 0B eB 1B","33":"2B"},H:{"2":"JC"},I:{"1":"G OC PC","2":"VB KC LC MC","8":"H NC eB"},J:{"1":"A","2":"D"},K:{"1":"Q","2":"A B C TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"1":"B","8":"A"},O:{"1":"QC"},P:{"1":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"1":"aC"},R:{"1":"bC"},S:{"1":"cC"}},B:1,C:"Mutation Observer"};
