module.exports={A:{A:{"2":"I D E F A B fB"},B:{"1":"R S T U V W X P Y Z G","2":"C J K L M N O"},C:{"1":"6 7 8 9 AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","16":"gB","33":"0 1 2 3 4 5 VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z iB jB"},D:{"1":"IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","16":"H a I D E F A B C J K","33":"0 1 2 3 4 5 6 7 8 9 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB"},E:{"1":"F A B C J K rB cB TB UB sB tB uB","16":"H a I nB bB oB","33":"D E pB qB"},F:{"1":"8 9 AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F B C vB wB xB yB TB dB zB UB","33":"0 1 2 3 4 5 6 7 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z"},G:{"1":"5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC","16":"bB 0B eB 1B","33":"E 2B 3B 4B"},H:{"2":"JC"},I:{"1":"G","16":"VB H KC LC MC NC eB","33":"OC PC"},J:{"16":"D A"},K:{"1":"Q","2":"A B C TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"33":"QC"},P:{"1":"VC cB WC XC YC ZC","16":"H","33":"RC SC TC UC"},Q:{"1":"aC"},R:{"1":"bC"},S:{"33":"cC"}},B:5,C:"CSS :any-link selector"};
