module.exports={A:{A:{"2":"I D E F fB","420":"A B"},B:{"2":"R S T U V W X P Y Z G","420":"C J K L M N O"},C:{"2":"0 1 2 3 4 5 6 7 8 9 gB VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G iB jB"},D:{"2":"0 1 2 3 4 5 6 7 8 9 H a I D E F A B C J K r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","36":"L M N O","66":"b c d e f g h i j k l m n o p q"},E:{"2":"H a I C J K nB bB oB TB UB sB tB uB","33":"D E F A B pB qB rB cB"},F:{"2":"0 1 2 3 4 5 6 7 8 9 F B C L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB vB wB xB yB TB dB zB UB"},G:{"2":"bB 0B eB 1B 2B AC BC CC DC EC FC GC HC IC","33":"E 3B 4B 5B 6B 7B 8B 9B"},H:{"2":"JC"},I:{"2":"VB H G KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"2":"A B C Q TB dB UB"},L:{"2":"G"},M:{"2":"P"},N:{"420":"A B"},O:{"2":"QC"},P:{"2":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"2":"aC"},R:{"2":"bC"},S:{"2":"cC"}},B:5,C:"CSS Regions"};
