module.exports={A:{A:{"2":"I D E F A B fB"},B:{"1":"N O R S T U V W X P Y Z G","2":"C J K L M"},C:{"2":"0 1 2 3 4 5 6 7 8 gB VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z iB jB","4609":"Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","4674":"XB","5698":"FB","7490":"9 AB BB CB DB","7746":"EB WB"},D:{"1":"KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"0 1 2 3 4 5 6 7 8 9 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB","4097":"JB","4290":"WB FB XB","6148":"Q GB HB IB"},E:{"2":"H a I D E F A nB bB oB pB qB rB cB","4609":"B C TB UB","8193":"J K sB tB uB"},F:{"1":"AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"0 1 2 3 4 F B C L M N O b c d e f g h i j k l m n o p q r s t u v w x y z vB wB xB yB TB dB zB UB","4097":"9","6148":"5 6 7 8"},G:{"1":"DC EC FC GC HC IC","2":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B 8B","4097":"9B AC BC CC"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"1":"Q","2":"A B C TB dB UB"},L:{"1":"G"},M:{"4097":"P"},N:{"2":"A B"},O:{"2":"QC"},P:{"2":"H RC SC TC","4097":"UC VC cB WC XC YC ZC"},Q:{"4097":"aC"},R:{"2":"bC"},S:{"2":"cC"}},B:5,C:"Variable fonts"};
