module.exports={A:{A:{"2":"I D E F A B fB"},B:{"1":"R S T U V W X P Y Z G","2":"C J K L","132":"M N O"},C:{"1":"8 9 AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"0 1 2 3 4 5 6 7 gB VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z iB jB"},D:{"1":"FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"0 1 2 3 4 5 6 7 8 9 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB","132":"DB EB WB"},E:{"1":"B C J K TB UB sB tB uB","2":"H a I D E F A nB bB oB pB qB rB","132":"cB"},F:{"1":"3 4 5 6 7 8 9 AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F B C L M N O b c d e f g h i j k l m n o p q r s t u v w x y z vB wB xB yB TB dB zB UB","132":"0 1 2"},G:{"1":"9B AC BC CC DC EC FC GC HC IC","2":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B","132":"8B"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"1":"Q","2":"A B C TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"132":"QC"},P:{"1":"UC VC cB WC XC YC ZC","2":"H RC SC","132":"TC"},Q:{"1":"aC"},R:{"2":"bC"},S:{"132":"cC"}},B:5,C:"CSS justify-content: space-evenly"};
