module.exports={A:{A:{"2":"I D E F A fB","388":"B"},B:{"1":"O R S T U V W","2":"C J K L","129":"M N","513":"X P Y Z G"},C:{"1":"FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"0 1 2 3 4 5 6 7 8 9 gB VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB iB jB"},D:{"1":"7 8 9 AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R","2":"0 1 2 3 4 5 6 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z","513":"S T U V W X P Y Z G kB lB mB"},E:{"1":"tB uB","2":"H a I D E F A B nB bB oB pB qB rB cB TB","2052":"K","3076":"C J UB sB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB","2":"F B C L M N O b c d e f g h i j k l m n o p q r s t u vB wB xB yB TB dB zB UB","513":"OB PB QB RB SB"},G:{"1":"DC EC FC GC HC IC","2":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B 8B 9B AC","2052":"BC CC"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"2":"A B C TB dB UB","513":"Q"},L:{"513":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"2":"QC"},P:{"1":"RC SC TC UC VC cB WC XC YC ZC","2":"H"},Q:{"16":"aC"},R:{"1":"bC"},S:{"2":"cC"}},B:6,C:"'SameSite' cookie attribute"};
