module.exports={A:{A:{"1":"B","2":"I D E F A fB"},B:{"1":"K L M N O R S T U V W X P Y Z G","129":"C J"},C:{"1":"6 7 8 9 AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB","260":"0 1 2 3 4 5 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z","804":"H a I D E F A B C J K iB jB"},D:{"1":"CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","260":"7 8 9 AB BB","388":"0 1 2 3 4 5 6 m n o p q r s t u v w x y z","1412":"L M N O b c d e f g h i j k l","1956":"H a I D E F A B C J K"},E:{"129":"A B C J K rB cB TB UB sB tB uB","1412":"I D E F pB qB","1956":"H a nB bB oB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F vB wB","260":"u v w x y","388":"L M N O b c d e f g h i j k l m n o p q r s t","1796":"xB yB","1828":"B C TB dB zB UB"},G:{"129":"6B 7B 8B 9B AC BC CC DC EC FC GC HC IC","1412":"E 2B 3B 4B 5B","1956":"bB 0B eB 1B"},H:{"1828":"JC"},I:{"1":"G","388":"OC PC","1956":"VB H KC LC MC NC eB"},J:{"1412":"A","1924":"D"},K:{"1":"Q","2":"A","1828":"B C TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"1":"B","2":"A"},O:{"388":"QC"},P:{"1":"TC UC VC cB WC XC YC ZC","260":"RC SC","388":"H"},Q:{"260":"aC"},R:{"260":"bC"},S:{"260":"cC"}},B:4,C:"CSS3 Border images"};
