module.exports={A:{A:{"1":"F A B","132":"I D E fB"},B:{"1":"K L M N O R S T U V W X P Y Z G","260":"C J"},C:{"1":"0 1 2 3 4 5 6 7 8 9 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","4":"gB VB iB jB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","4":"H"},E:{"1":"a I D E F A B C J K oB pB qB rB cB TB UB sB tB uB","4":"H nB bB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","260":"F B C vB wB xB yB TB dB zB UB"},G:{"2":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D","16":"A"},K:{"1":"Q","2":"A B C TB dB UB"},L:{"1":"G"},M:{"2":"P"},N:{"2":"A B"},O:{"2":"QC"},P:{"2":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"2":"aC"},R:{"2":"bC"},S:{"2":"cC"}},B:4,C:"CSS3 Cursors (original values)"};
