module.exports={A:{A:{"2":"fB","8":"I D E","772":"F A B"},B:{"1":"R S T U V W X P Y Z G","513":"C J K L M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G iB jB","4":"gB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB"},E:{"1":"H a I D E F A B C J K bB oB pB qB rB cB TB UB sB tB uB","4":"nB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 F B C L M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB vB wB xB yB TB dB zB UB"},G:{"1":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC"},H:{"1":"JC"},I:{"1":"G OC PC","2":"KC LC MC","132":"VB H NC eB"},J:{"1":"D A"},K:{"1":"A B C Q TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"257":"A B"},O:{"1":"QC"},P:{"1":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"1":"aC"},R:{"1":"bC"},S:{"1":"cC"}},B:4,C:"SVG (basic support)"};
