module.exports={A:{A:{"2":"I D E F A B fB"},B:{"2":"C J K L M N","260":"R S T U V W X P Y Z G","3138":"O"},C:{"1":"AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB","132":"0 1 2 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z iB jB","644":"3 4 5 6 7 8 9"},D:{"2":"H a I D E F A B C J K L M N O b c d e f","260":"BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","292":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w x y z AB"},E:{"2":"H a I nB bB oB pB","292":"D E F A B C J K qB rB cB TB UB sB tB uB"},F:{"2":"F B C vB wB xB yB TB dB zB UB","260":"0 1 2 3 4 5 6 7 8 9 y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","292":"L M N O b c d e f g h i j k l m n o p q r s t u v w x"},G:{"2":"bB 0B eB 1B 2B","292":"E 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC"},H:{"2":"JC"},I:{"2":"VB H KC LC MC NC eB","260":"G","292":"OC PC"},J:{"2":"D A"},K:{"2":"A B C TB dB UB","260":"Q"},L:{"260":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"292":"QC"},P:{"292":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"292":"aC"},R:{"260":"bC"},S:{"644":"cC"}},B:4,C:"CSS clip-path property (for HTML)"};
