module.exports={A:{A:{"2":"I D E F A B fB"},B:{"1":"R S T U V W X P Y Z G","2":"C J K L M N O"},C:{"2":"gB VB iB jB","33":"9 AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","164":"0 1 2 3 4 5 6 7 8 H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y z"},D:{"1":"0 1 2 3 4 5 6 7 8 9 y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"H a I D E F A B C J K L M N O b c","132":"d e f g h i j k l m n o p q r s t u v w x"},E:{"1":"K sB tB uB","2":"H a I nB bB oB","132":"D E F A B C J pB qB rB cB TB UB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F vB wB xB","132":"L M N O b c d e f g h i j k","164":"B C yB TB dB zB UB"},G:{"1":"GC HC IC","2":"bB 0B eB 1B 2B","132":"E 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC"},H:{"164":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB","132":"OC PC"},J:{"132":"D A"},K:{"1":"Q","2":"A","164":"B C TB dB UB"},L:{"1":"G"},M:{"33":"P"},N:{"2":"A B"},O:{"1":"QC"},P:{"1":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"1":"aC"},R:{"1":"bC"},S:{"164":"cC"}},B:5,C:"CSS3 tab-size"};
