module.exports={A:{A:{"1":"B","2":"I D E F fB","164":"A"},B:{"1":"C J K L M N O R S T U V W X P Y Z G"},C:{"1":"WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB H a iB jB","8":"I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w","328":"0 1 2 3 4 5 6 7 8 9 x y z AB BB CB DB EB"},D:{"1":"BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"H a I D E F A B C J K L M N O b c d","8":"0 1 2 3 4 5 6 7 e f g h i j k l m n o p q r s t u v w x y z","584":"8 9 AB"},E:{"1":"J K sB tB uB","2":"H a I nB bB oB","8":"D E F A B C pB qB rB cB TB","1096":"UB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F B C vB wB xB yB TB dB zB UB","8":"L M N O b c d e f g h i j k l m n o p q r s t u","584":"v w x"},G:{"1":"EC FC GC HC IC","8":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC","6148":"DC"},H:{"2":"JC"},I:{"1":"G","8":"VB H KC LC MC NC eB OC PC"},J:{"8":"D A"},K:{"1":"Q","2":"A","8":"B C TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"1":"B","36":"A"},O:{"8":"QC"},P:{"1":"SC TC UC VC cB WC XC YC ZC","2":"RC","8":"H"},Q:{"1":"aC"},R:{"2":"bC"},S:{"328":"cC"}},B:2,C:"Pointer events"};
