module.exports={A:{A:{"2":"I D E F A B fB"},B:{"2":"C J K L M N O","2052":"R S T U V W X P Y Z G"},C:{"2":"gB VB H a iB jB","1028":"0 1 2 3 4 5 6 7 8 9 s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","1060":"I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r"},D:{"2":"H a I D E F A B C J K L M N O b c d e f g h","226":"0 1 2 3 4 5 6 7 8 9 i j k l m n o p q r s t u v w x y z AB BB CB","2052":"DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB"},E:{"2":"H a I D nB bB oB pB","772":"J K UB sB tB uB","804":"E F A B C rB cB TB","1316":"qB"},F:{"2":"F B C L M N O b c d e f g h i j k l m n o p q vB wB xB yB TB dB zB UB","226":"r s t u v w x y z","2052":"0 1 2 3 4 5 6 7 8 9 AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB"},G:{"2":"bB 0B eB 1B 2B 3B","292":"E 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"2":"A B C TB dB UB","2052":"Q"},L:{"2052":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"2052":"QC"},P:{"2":"H RC SC","2052":"TC UC VC cB WC XC YC ZC"},Q:{"2":"aC"},R:{"1":"bC"},S:{"1028":"cC"}},B:4,C:"text-decoration styling"};
