module.exports={A:{A:{"2":"I D E F A B fB"},B:{"1":"V W X P Y Z G","2":"C J K L M N O","260":"R S T U"},C:{"1":"T hB U V W X P Y Z G","2":"gB VB H a I D E F A B C J K L M N O b c d e f g h i j k l m n o iB jB","260":"WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB","516":"3 4 5 6 7 8 9 AB BB CB DB EB","580":"0 1 2 p q r s t u v w x y z","2049":"SB YB ZB aB R S"},D:{"1":"V W X P Y Z G kB lB mB","2":"H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r","132":"s t u","260":"0 1 2 3 4 5 6 7 8 9 v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U"},E:{"1":"uB","2":"H a I D E F A nB bB oB pB qB rB cB","1090":"B C J TB UB","2049":"K sB tB"},F:{"1":"OB PB QB RB SB","2":"F B C L M N O b c d e vB wB xB yB TB dB zB UB","132":"f g h","260":"0 1 2 3 4 5 6 7 8 9 i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB"},G:{"2":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B 8B","1090":"9B AC BC CC DC EC FC","2049":"GC HC IC"},H:{"2":"JC"},I:{"1":"G","2":"VB H KC LC MC NC eB OC PC"},J:{"2":"D A"},K:{"1":"Q","2":"A B C TB dB UB"},L:{"1":"G"},M:{"1":"P"},N:{"2":"A B"},O:{"260":"QC"},P:{"260":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"260":"aC"},R:{"260":"bC"},S:{"516":"cC"}},B:5,C:"Web Animations API"};
