module.exports={A:{A:{"2":"I D E fB","520":"F A B"},B:{"1":"R S T U V W X P Y Z G","8":"C J","388":"K L M N O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB iB jB","132":"H a I D E F A B C J K L M N O b c d e f g h i j"},D:{"1":"0 1 2 3 4 5 6 7 8 9 h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","2":"H a","132":"I D E F A B C J K L M N O b c d e f g"},E:{"2":"nB","8":"H a bB oB","520":"I D E F A B C pB qB rB cB TB","1028":"J UB sB","7172":"K","8196":"tB uB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB","2":"F vB wB xB","132":"B C L yB TB dB zB UB"},G:{"2":"E bB 0B eB 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC","1028":"CC DC EC FC GC","3076":"HC IC"},H:{"2":"JC"},I:{"1":"G","2":"KC LC","132":"VB H MC NC eB OC PC"},J:{"2":"D A"},K:{"2":"A B C TB dB UB","132":"Q"},L:{"1":"G"},M:{"1":"P"},N:{"8":"A B"},O:{"1":"QC"},P:{"1":"RC SC TC UC VC cB WC XC YC ZC","132":"H"},Q:{"1":"aC"},R:{"1":"bC"},S:{"1":"cC"}},B:6,C:"WebM video format"};
