module.exports={A:{A:{"1":"A B","2":"I D E F fB"},B:{"1":"C J K L M N O R S T U V W X P Y Z G"},C:{"1":"0 1 2 3 4 5 6 7 8 9 M N O b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T hB U V W X P Y Z G","2":"gB VB H iB jB","33":"a I D E F A B C J K L"},D:{"1":"0 1 2 3 4 5 6 7 8 9 z AB BB CB DB EB WB FB XB Q GB HB IB JB KB LB MB NB OB PB QB RB SB YB ZB aB R S T U V W X P Y Z G kB lB mB","33":"H a I D E F A B C J K L M N O b c d e f g h i j k l m n o p q r s t u v w x y"},E:{"1":"F A B C J K rB cB TB UB sB tB uB","2":"nB bB","33":"I D E oB pB qB","292":"H a"},F:{"1":"0 1 2 3 4 5 6 7 8 9 m n o p q r s t u v w x y z AB BB CB DB EB FB Q GB HB IB JB KB LB MB NB OB PB QB RB SB UB","2":"F B vB wB xB yB TB dB zB","33":"C L M N O b c d e f g h i j k l"},G:{"1":"5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC IC","33":"E 2B 3B 4B","164":"bB 0B eB 1B"},H:{"2":"JC"},I:{"1":"G","33":"H NC eB OC PC","164":"VB KC LC MC"},J:{"33":"D A"},K:{"1":"Q UB","2":"A B C TB dB"},L:{"1":"G"},M:{"1":"P"},N:{"1":"A B"},O:{"1":"QC"},P:{"1":"H RC SC TC UC VC cB WC XC YC ZC"},Q:{"33":"aC"},R:{"1":"bC"},S:{"1":"cC"}},B:5,C:"CSS Animation"};
